@@ -7,9 +7,8 @@ from fedn.network.api.client import APIClient
 # flake8: noqa
 
 
-modules = glob.glob(dirname(__file__) + "/*.py")
-__all__ = [basename(f)[:-3] for f in modules if isfile(f)
-           and not f.endswith('__init__.py')]
+modules = glob.glob(dirname(__file__) + "/fedn" + "/*.py")
+__all__ = [basename(f)[:-3] for f in modules if isfile(f) and not f.endswith("__init__.py")]
 
 
 _ROOT = os.path.abspath(os.path.dirname(__file__))
@@ -21,4 +20,4 @@ def get_data(path):
     :param path:
     :return:
     """
-    return os.path.join(_ROOT, 'data', path)
+    return os.path.join(_ROOT, "data", path)
