@@ -1,20 +1,18 @@
 import base64
 import copy
-import os
 import threading
 import uuid
 from io import BytesIO
 
 from flask import jsonify, send_from_directory
+from werkzeug.security import safe_join
 from werkzeug.utils import secure_filename
 
 from fedn.common.config import get_controller_config, get_network_config
 from fedn.common.log_config import logger
-from fedn.network.combiner.interfaces import (CombinerInterface,
-                                              CombinerUnavailableError)
+from fedn.network.combiner.interfaces import CombinerInterface, CombinerUnavailableError
 from fedn.network.state import ReducerState, ReducerStateToString
 from fedn.utils.checksum import sha
-from fedn.utils.plots import Plot
 
 __all__ = ("API",)
 
@@ -36,9 +34,7 @@ class API:
         data = {"name": self.name}
         return data
 
-    def _allowed_file_extension(
-        self, filename, ALLOWED_EXTENSIONS={"gz", "bz2", "tar", "zip", "tgz"}
-    ):
+    def _allowed_file_extension(self, filename, ALLOWED_EXTENSIONS={"gz", "bz2", "tar", "zip", "tgz"}):
         """Check if file extension is allowed.
 
         :param filename: The filename to check.
@@ -170,11 +166,10 @@ class API:
         info = session_object["session_config"][0]
         status = session_object["status"]
         payload[id] = info
-        payload['status'] = status
+        payload["status"] = status
         return jsonify(payload)
 
     def set_active_compute_package(self, id: str):
-
         success = self.statestore.set_active_compute_package(id)
 
         if not success:
@@ -198,17 +193,12 @@ class API:
         :return: A json response with success or failure message.
         :rtype: :class:`flask.Response`
         """
-
-        if (
-            self.control.state() == ReducerState.instructing
-            or self.control.state() == ReducerState.monitoring
-        ):
+        if self.control.state() == ReducerState.instructing or self.control.state() == ReducerState.monitoring:
             return (
                 jsonify(
                     {
                         "success": False,
-                        "message": "Reducer is in instructing or monitoring state."
-                        "Cannot set compute package.",
+                        "message": "Reducer is in instructing or monitoring state." "Cannot set compute package.",
                     }
                 ),
                 400,
@@ -241,7 +231,7 @@ class API:
         file_name = file.filename
         storage_file_name = secure_filename(f"{str(uuid.uuid4())}.{extension}")
 
-        file_path = os.path.join("/app/client/package/", storage_file_name)
+        file_path = safe_join("/app/client/package/", storage_file_name)
         file.save(file_path)
 
         self.control.set_compute_package(storage_file_name, file_path)
@@ -288,9 +278,7 @@ class API:
         result = self.statestore.get_compute_package()
         if result is None:
             return (
-                jsonify(
-                    {"success": False, "message": "No compute package found."}
-                ),
+                jsonify({"success": False, "message": "No compute package found."}),
                 404,
             )
 
@@ -317,7 +305,6 @@ class API:
         :return: All compute packages as a json response.
         :rtype: :class:`flask.Response`
         """
-
         if limit is not None and skip is not None:
             limit = int(limit)
             skip = int(skip)
@@ -327,9 +314,7 @@ class API:
         result = self.statestore.list_compute_packages(limit, skip)
         if result is None:
             return (
-                jsonify(
-                    {"success": False, "message": "No compute packages found."}
-                ),
+                jsonify({"success": False, "message": "No compute packages found."}),
                 404,
             )
 
@@ -386,20 +371,16 @@ class API:
             mutex = threading.Lock()
             mutex.acquire()
             # TODO: make configurable, perhaps in config.py or package.py
-            return send_from_directory(
-                "/app/client/package/", name, as_attachment=True
-            )
+            return send_from_directory("/app/client/package/", name, as_attachment=True)
         except Exception:
             try:
                 data = self.control.get_compute_package(name)
                 # TODO: make configurable, perhaps in config.py or package.py
-                file_path = os.path.join("/app/client/package/", name)
+                file_path = safe_join("/app/client/package/", name)
                 with open(file_path, "wb") as fh:
                     fh.write(data)
                 # TODO: make configurable, perhaps in config.py or package.py
-                return send_from_directory(
-                    "/app/client/package/", name, as_attachment=True
-                )
+                return send_from_directory("/app/client/package/", name, as_attachment=True)
             except Exception:
                 raise
         finally:
@@ -413,14 +394,11 @@ class API:
         :return: Success or failure boolean, message and the checksum.
         :rtype: bool, str, str
         """
-
         if name is None:
             name, message = self._get_compute_package_name()
             if name is None:
                 return False, message, ""
-        file_path = os.path.join(
-            "/app/client/package/", name
-        )  # TODO: make configurable, perhaps in config.py or package.py
+        file_path = safe_join("/app/client/package/", name)  # TODO: make configurable, perhaps in config.py or package.py
         try:
             sum = str(sha(file_path))
         except FileNotFoundError:
@@ -436,7 +414,6 @@ class API:
         :return: The checksum as a json object.
         :rtype: :py:class:`flask.Response`
         """
-
         success, message, sum = self._create_checksum(name)
         if not success:
             return jsonify({"success": False, "message": message}), 404
@@ -505,9 +482,7 @@ class API:
             payload[id] = info
         return jsonify(payload)
 
-    def add_combiner(
-        self, combiner_id, secure_grpc, address, remote_addr, fqdn, port
-    ):
+    def add_combiner(self, combiner_id, secure_grpc, address, remote_addr, fqdn, port):
         """Add a combiner to the network.
 
         :param combiner_id: The combiner id to add.
@@ -540,9 +515,7 @@ class API:
         combiner = self.control.network.get_combiner(combiner_id)
         if not combiner:
             if secure_grpc == "True":
-                certificate, key = self.certificate_manager.get_or_create(
-                    address
-                ).get_keypair_raw()
+                certificate, key = self.certificate_manager.get_or_create(address).get_keypair_raw()
                 _ = base64.b64encode(certificate)
                 _ = base64.b64encode(key)
 
@@ -566,9 +539,7 @@ class API:
         # Check combiner now exists
         combiner = self.control.network.get_combiner(combiner_id)
         if not combiner:
-            return jsonify(
-                {"success": False, "message": "Combiner not added."}
-            )
+            return jsonify({"success": False, "message": "Combiner not added."})
 
         payload = {
             "success": True,
@@ -582,7 +553,7 @@ class API:
 
         return jsonify(payload)
 
-    def add_client(self, client_id, preferred_combiner, remote_addr):
+    def add_client(self, client_id, preferred_combiner, remote_addr, name):
         """Add a client to the network.
 
         :param client_id: The client id to add.
@@ -623,14 +594,13 @@ class API:
             combiner = self.control.network.find_available_combiner()
             if combiner is None:
                 return (
-                    jsonify(
-                        {"success": False, "message": "No combiner available."}
-                    ),
+                    jsonify({"success": False, "message": "No combiner available."}),
                     400,
                 )
 
         client_config = {
-            "name": client_id,
+            "client_id": client_id,
+            "name": name,
             "combiner_preferred": preferred_combiner,
             "combiner": combiner.name,
             "ip": remote_addr,
@@ -656,8 +626,6 @@ class API:
             "certificate": cert,
             "helper_type": self.control.statestore.get_helper(),
         }
-        logger.info(f"Sending payload: {payload}")
-
         return jsonify(payload)
 
     def get_initial_model(self):
@@ -689,11 +657,9 @@ class API:
             self.control.commit(file.filename, model)
         except Exception as e:
             logger.debug(e)
-            return jsonify({"success": False, "message": e})
+            return jsonify({"success": False, "message": "Failed to add initial model."})
 
-        return jsonify(
-            {"success": True, "message": "Initial model added successfully."}
-        )
+        return jsonify({"success": True, "message": "Initial model added successfully."})
 
     def get_latest_model(self):
         """Get the latest model from the statestore.
@@ -706,9 +672,7 @@ class API:
             payload = {"model_id": model_id}
             return jsonify(payload)
         else:
-            return jsonify(
-                {"success": False, "message": "No initial model set."}
-            )
+            return jsonify({"success": False, "message": "No initial model set."})
 
     def set_current_model(self, model_id: str):
         """Set the active model in the statestore.
@@ -745,7 +709,6 @@ class API:
         include_active: bool = include_active == "true"
 
         if include_active:
-
             latest_model = self.statestore.get_latest_model()
 
             arr = [
@@ -801,9 +764,7 @@ class API:
         if model_info:
             return jsonify(model_info)
         else:
-            return jsonify(
-                {"success": False, "message": "No model trail available."}
-            )
+            return jsonify({"success": False, "message": "No model trail available."})
 
     def get_model_ancestors(self, model_id: str, limit: str = None):
         """Get the model ancestors for a given model.
@@ -816,15 +777,12 @@ class API:
         :rtype: :class:`flask.Response`
         """
         if model_id is None:
-            return jsonify(
-                {"success": False, "message": "No model id provided."}
-            )
+            return jsonify({"success": False, "message": "No model id provided."})
 
         limit: int = int(limit) if limit is not None else 10  # if limit is None, default to 10
 
         response = self.statestore.get_model_ancestors(model_id, limit)
         if response:
-
             arr: list = []
 
             for element in response:
@@ -840,9 +798,7 @@ class API:
 
             return jsonify(result)
         else:
-            return jsonify(
-                {"success": False, "message": "No model ancestors available."}
-            )
+            return jsonify({"success": False, "message": "No model ancestors available."})
 
     def get_model_descendants(self, model_id: str, limit: str = None):
         """Get the model descendants for a given model.
@@ -854,18 +810,14 @@ class API:
         :return: The model descendants for the given model as a json response.
         :rtype: :class:`flask.Response`
         """
-
         if model_id is None:
-            return jsonify(
-                {"success": False, "message": "No model id provided."}
-            )
+            return jsonify({"success": False, "message": "No model id provided."})
 
         limit: int = int(limit) if limit is not None else 10
 
         response: list = self.statestore.get_model_descendants(model_id, limit)
 
         if response:
-
             arr: list = []
 
             for element in response:
@@ -881,9 +833,7 @@ class API:
 
             return jsonify(result)
         else:
-            return jsonify(
-                {"success": False, "message": "No model descendants available."}
-            )
+            return jsonify({"success": False, "message": "No model descendants available."})
 
     def get_all_rounds(self):
         """Get all rounds.
@@ -911,8 +861,7 @@ class API:
                 "combiners": combiners,
             }
             payload[id] = info
-        else:
-            return jsonify(payload)
+        return jsonify(payload)
 
     def get_round(self, round_id):
         """Get a round.
@@ -926,8 +875,8 @@ class API:
         if round_object is None:
             return jsonify({"success": False, "message": "Round not found."})
         payload = {
-            'round_id': round_object['round_id'],
-            'combiners': round_object['combiners'],
+            "round_id": round_object["round_id"],
+            "combiners": round_object["combiners"],
         }
         return jsonify(payload)
 
@@ -952,31 +901,6 @@ class API:
                 payload["checksum"] = checksum_str
         return jsonify(payload)
 
-    def get_plot_data(self, feature=None):
-        """Get plot data.
-
-        :return: The plot data as json response.
-        :rtype: :py:class:`flask.Response`
-        """
-
-        plot = Plot(self.control.statestore)
-
-        try:
-            valid_metrics = plot.fetch_valid_metrics()
-            feature = feature or valid_metrics[0]
-            box_plot = plot.create_box_plot(feature)
-        except Exception as e:
-            valid_metrics = None
-            box_plot = None
-            logger.debug(e)
-
-        result = {
-            "valid_metrics": valid_metrics,
-            "box_plot": box_plot,
-        }
-
-        return jsonify(result)
-
     def list_combiners_data(self, combiners):
         """Get combiners data.
 
@@ -985,14 +909,12 @@ class API:
         :return: The combiners data as json response.
         :rtype: :py:class:`flask.Response`
         """
-
         response = self.statestore.list_combiners_data(combiners)
 
         arr = []
 
         # order list by combiner name
         for element in response:
-
             obj = {
                 "combiner": element["_id"],
                 "count": element["count"],
@@ -1007,14 +929,15 @@ class API:
     def start_session(
         self,
         session_id,
-        aggregator='fedavg',
+        aggregator="fedavg",
+        aggregator_kwargs=None,
         model_id=None,
         rounds=5,
         round_timeout=180,
         round_buffer_size=-1,
         delete_models=True,
         validate=True,
-        helper="numpyhelper",
+        helper="",
         min_clients=1,
         requested_clients=8,
     ):
@@ -1046,24 +969,24 @@ class API:
         # Check if session already exists
         session = self.statestore.get_session(session_id)
         if session:
-            return jsonify(
-                {"success": False, "message": "Session already exists."}
-            )
+            return jsonify({"success": False, "message": "Session already exists."})
 
         # Check if session is running
         if self.control.state() == ReducerState.monitoring:
-            return jsonify(
-                {"success": False, "message": "A session is already running."}
-            )
+            return jsonify({"success": False, "message": "A session is already running."})
 
         # Check if compute package is set
-        if not self.statestore.get_compute_package():
+        package = self.statestore.get_compute_package()
+        if not package:
             return jsonify(
                 {
                     "success": False,
                     "message": "No compute package set. Set compute package before starting session.",
                 }
             )
+        if not helper:
+            # get helper from compute package
+            helper = package["helper"]
 
         # Check that initial (seed) model is set
         if not self.statestore.get_initial_model():
@@ -1108,6 +1031,7 @@ class API:
         session_config = {
             "session_id": session_id if session_id else str(uuid.uuid4()),
             "aggregator": aggregator,
+            "aggregator_kwargs": aggregator_kwargs,
             "round_timeout": round_timeout,
             "buffer_size": round_buffer_size,
             "model_id": model_id,
@@ -1121,9 +1045,7 @@ class API:
         }
 
         # Start session
-        threading.Thread(
-            target=self.control.session, args=(session_config,)
-        ).start()
+        threading.Thread(target=self.control.session, args=(session_config,)).start()
 
         # Return success response
         return jsonify(
