@@ -2,35 +2,25 @@ import os
 
 from flask import Flask, jsonify, request
 
-from fedn.common.config import (get_controller_config, get_modelstorage_config,
-                                get_network_config, get_statestore_config)
+from fedn.common.config import get_controller_config
 from fedn.network.api.auth import jwt_auth_required
 from fedn.network.api.interface import API
+from fedn.network.api.shared import control, statestore
 from fedn.network.api.v1 import _routes
-from fedn.network.controller.control import Control
-from fedn.network.storage.statestore.mongostatestore import MongoStateStore
-
-statestore_config = get_statestore_config()
-network_id = get_network_config()
-modelstorage_config = get_modelstorage_config()
-statestore = MongoStateStore(network_id, statestore_config["mongo_config"])
-statestore.set_storage_backend(modelstorage_config)
-control = Control(statestore=statestore)
 
 custom_url_prefix = os.environ.get("FEDN_CUSTOM_URL_PREFIX", False)
+# statestore_config,modelstorage_config,network_id,control=set_statestore_config()
 api = API(statestore, control)
 app = Flask(__name__)
 for bp in _routes:
     app.register_blueprint(bp)
     if custom_url_prefix:
-        app.register_blueprint(bp,
-                               name=f"{bp.name}_custom",
-                               url_prefix=f"{custom_url_prefix}{bp.url_prefix}")
+        app.register_blueprint(bp, name=f"{bp.name}_custom", url_prefix=f"{custom_url_prefix}{bp.url_prefix}")
 
 
-@app.route('/health', methods=["GET"])
+@app.route("/health", methods=["GET"])
 def health_check():
-    return 'OK', 200
+    return "OK", 200
 
 
 if custom_url_prefix:
@@ -108,7 +98,6 @@ def list_models():
     Returns:
         _type_: json
     """
-
     session_id = request.args.get("session_id", None)
     limit = request.args.get("limit", None)
     skip = request.args.get("skip", None)
@@ -164,7 +153,6 @@ def list_clients():
     return: All clients as a json object.
     rtype: json
     """
-
     limit = request.args.get("limit", None)
     skip = request.args.get("skip", None)
     status = request.args.get("status", None)
@@ -205,7 +193,6 @@ def list_combiners():
     return: All combiners as a json object.
     rtype: json
     """
-
     limit = request.args.get("limit", None)
     skip = request.args.get("skip", None)
 
@@ -364,9 +351,7 @@ def set_package():
         file = request.files["file"]
     except KeyError:
         return jsonify({"success": False, "message": "Missing file."}), 400
-    return api.set_compute_package(
-        file=file, helper_type=helper_type, name=name, description=description
-    )
+    return api.set_compute_package(file=file, helper_type=helper_type, name=name, description=description)
 
 
 if custom_url_prefix:
@@ -394,14 +379,11 @@ def list_compute_packages():
     return: The compute package as a json object.
     rtype: json
     """
-
     limit = request.args.get("limit", None)
     skip = request.args.get("skip", None)
     include_active = request.args.get("include_active", None)
 
-    return api.list_compute_packages(
-        limit=limit, skip=skip, include_active=include_active
-    )
+    return api.list_compute_packages(limit=limit, skip=skip, include_active=include_active)
 
 
 if custom_url_prefix:
@@ -587,8 +569,10 @@ def add_combiner():
     remote_addr = request.remote_addr
     try:
         response = api.add_combiner(**json_data, remote_addr=remote_addr)
-    except TypeError as e:
-        return jsonify({"success": False, "message": str(e)}), 400
+    except TypeError:
+        return jsonify({"success": False, "message": "Invalid data provided"}), 400
+    except Exception:
+        return jsonify({"success": False, "message": "An unexpected error occurred"}), 500
     return response
 
 
@@ -603,13 +587,14 @@ def add_client():
     return: The response from control.
     rtype: json
     """
-
     json_data = request.get_json()
     remote_addr = request.remote_addr
     try:
         response = api.add_client(**json_data, remote_addr=remote_addr)
-    except TypeError as e:
-        return jsonify({"success": False, "message": str(e)}), 400
+    except TypeError:
+        return jsonify({"success": False, "message": "Invalid data provided"}), 400
+    except Exception:
+        return jsonify({"success": False, "message": "An unexpected error occurred"}), 500
     return response
 
 
@@ -624,7 +609,6 @@ def list_combiners_data():
     return: The response from control.
     rtype: json
     """
-
     json_data = request.get_json()
 
     # expects a list of combiner names (strings) in an array
@@ -632,8 +616,10 @@ def list_combiners_data():
 
     try:
         response = api.list_combiners_data(combiners)
-    except TypeError as e:
-        return jsonify({"success": False, "message": str(e)}), 400
+    except TypeError:
+        return jsonify({"success": False, "message": "Invalid data provided"}), 400
+    except Exception:
+        return jsonify({"success": False, "message": "An unexpected error occurred"}), 500
     return response
 
 
@@ -641,26 +627,13 @@ if custom_url_prefix:
     app.add_url_rule(f"{custom_url_prefix}/list_combiners_data", view_func=list_combiners_data, methods=["POST"])
 
 
-@app.route("/get_plot_data", methods=["GET"])
-@jwt_auth_required(role="admin")
-def get_plot_data():
-    """Get plot data from the statestore.
-    rtype: json
-    """
-
-    try:
-        feature = request.args.get("feature", None)
-        response = api.get_plot_data(feature=feature)
-    except TypeError as e:
-        return jsonify({"success": False, "message": str(e)}), 400
-    return response
-
-
-if custom_url_prefix:
-    app.add_url_rule(f"{custom_url_prefix}/get_plot_data", view_func=get_plot_data, methods=["GET"])
-
-if __name__ == "__main__":
+def start_server_api():
     config = get_controller_config()
     port = config["port"]
     debug = config["debug"]
-    app.run(debug=debug, port=port, host="0.0.0.0")
+    host = "0.0.0.0"
+    app.run(debug=debug, port=port, host=host)
+
+
+if __name__ == "__main__":
+    start_server_api()
