@@ -4,10 +4,8 @@ import numpy as np
 from flask import Blueprint, jsonify, request, send_file
 
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.v1.shared import (api_version, get_limit,
-                                        get_post_data_to_kwargs, get_reverse,
-                                        get_typed_list_headers, mdb,
-                                        modelstorage_config)
+from fedn.network.api.shared import modelstorage_config
+from fedn.network.api.v1.shared import api_version, get_limit, get_post_data_to_kwargs, get_reverse, get_typed_list_headers, mdb
 from fedn.network.storage.s3.base import RepositoryBase
 from fedn.network.storage.s3.miniorepository import MINIORepository
 from fedn.network.storage.statestore.stores.model_store import ModelStore
@@ -112,17 +110,15 @@ def get_models():
         limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
         kwargs = request.args.to_dict()
 
-        models = model_store.list(
-            limit, skip, sort_key, sort_order, use_typing=False, **kwargs
-        )
+        models = model_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
 
         result = models["result"]
 
         response = {"count": models["count"], "result": result}
 
         return jsonify(response), 200
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/list", methods=["POST"])
@@ -199,17 +195,15 @@ def list_models():
         limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
         kwargs = get_post_data_to_kwargs(request)
 
-        models = model_store.list(
-            limit, skip, sort_key, sort_order, use_typing=False, **kwargs
-        )
+        models = model_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
 
         result = models["result"]
 
         response = {"count": models["count"], "result": result}
 
         return jsonify(response), 200
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/count", methods=["GET"])
@@ -256,8 +250,8 @@ def get_models_count():
         count = model_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/count", methods=["POST"])
@@ -308,8 +302,8 @@ def models_count():
         count = model_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/<string:id>", methods=["GET"])
@@ -352,10 +346,132 @@ def get_model(id: str):
         response = model
 
         return jsonify(response), 200
-    except EntityNotFound as e:
-        return jsonify({"message": str(e)}), 404
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/<string:id>", methods=["PATCH"])
+@jwt_auth_required(role="admin")
+def patch_model(id: str):
+    """Patch model
+    Updates a model based on the provided id. Only the fields that are present in the request will be updated.
+    ---
+    tags:
+        - Models
+    parameters:
+        - name: id
+            in: path
+            required: true
+            type: string
+            description: The id or model property of the model
+        - name: model
+            in: body
+            required: true
+            type: object
+            description: The model data to update
+    responses:
+        200:
+            description: The updated model
+            schema:
+                $ref: '#/definitions/Model'
+        404:
+            description: The model was not found
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+        500:
+            description: An error occurred
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+    """
+    try:
+        model = model_store.get(id, use_typing=False)
+
+        data = request.get_json()
+        _id = model["id"]
+
+        # Update the model with the new data
+        # Only update the fields that are present in the request
+        for key, value in data.items():
+            if key in ["_id", "model"]:
+                continue
+            model[key] = value
+
+        success, message = model_store.update(_id, model)
+
+        if success:
+            response = model
+            return jsonify(response), 200
+
+        return jsonify({"message": f"Failed to update model: {message}"}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/<string:id>", methods=["PUT"])
+@jwt_auth_required(role="admin")
+def put_model(id: str):
+    """Put model
+    Updates a model based on the provided id. All fields will be updated with the new data.
+    ---
+    tags:
+        - Models
+    parameters:
+        - name: id
+            in: path
+            required: true
+            type: string
+            description: The id or model property of the model
+        - name: model
+            in: body
+            required: true
+            type: object
+            description: The model data to update
+    responses:
+        200:
+            description: The updated model
+            schema:
+                $ref: '#/definitions/Model'
+        404:
+            description: The model was not found
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+        500:
+            description: An error occurred
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+    """
+    try:
+        model = model_store.get(id, use_typing=False)
+        data = request.get_json()
+        _id = model["id"]
+
+        success, message = model_store.update(_id, data)
+
+        if success:
+            response = model
+            return jsonify(response), 200
+
+        return jsonify({"message": f"Failed to update model: {message}"}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/<string:id>/descendants", methods=["GET"])
@@ -406,10 +522,10 @@ def get_descendants(id: str):
         response = descendants
 
         return jsonify(response), 200
-    except EntityNotFound as e:
-        return jsonify({"message": str(e)}), 404
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/<string:id>/ancestors", methods=["GET"])
@@ -466,7 +582,7 @@ def get_ancestors(id: str):
     try:
         limit = get_limit(request.headers)
         reverse = get_reverse(request.headers)
-        include_self_param: str = request.args.get('include_self')
+        include_self_param: str = request.args.get("include_self")
 
         include_self: bool = include_self_param and include_self_param.lower() == "true"
 
@@ -475,13 +591,14 @@ def get_ancestors(id: str):
         response = ancestors
 
         return jsonify(response), 200
-    except EntityNotFound as e:
-        return jsonify({"message": str(e)}), 404
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/<string:id>/download", methods=["GET"])
+@jwt_auth_required(role="admin")
 def download(id: str):
     """Download
     Downloads the model file of the provided id.
@@ -523,13 +640,14 @@ def download(id: str):
             return send_file(file, as_attachment=True, download_name=model_id)
         else:
             return jsonify({"message": "No model storage configured"}), 500
-    except EntityNotFound as e:
-        return jsonify({"message": str(e)}), 404
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
 
 @bp.route("/<string:id>/parameters", methods=["GET"])
+@jwt_auth_required(role="admin")
 def get_parameters(id: str):
     """Download
     Downloads parameters of the model of the provided id.
@@ -587,7 +705,40 @@ def get_parameters(id: str):
             return jsonify(array=weights), 200
         else:
             return jsonify({"message": "No model storage configured"}), 500
-    except EntityNotFound as e:
-        return jsonify({"message": str(e)}), 404
-    except Exception as e:
-        return jsonify({"message": str(e)}), 500
+    except EntityNotFound:
+        return jsonify({"message": f"Entity with id: {id} not found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/active", methods=["GET"])
+@jwt_auth_required(role="admin")
+def get_active_model():
+    """Get active model
+    Retrieves the active model (id).
+    ---
+    tags:
+        - Models
+    responses:
+        200:
+            description: The active model id
+            schema:
+                type: string
+        500:
+            description: An error occurred
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+    """
+    try:
+        active_model = model_store.get_active()
+
+        response = active_model
+
+        return jsonify(response), 200
+    except EntityNotFound:
+        return jsonify({"message": "No active model found"}), 404
+    except Exception:
+        return jsonify({"message": "An unexpected error occurred"}), 500
