@@ -12,20 +12,20 @@ from enum import Enum
 
 import fedn.network.grpc.fedn_pb2 as fedn
 import fedn.network.grpc.fedn_pb2_grpc as rpc
-from fedn.common.log_config import (logger, set_log_level_from_string,
-                                    set_log_stream)
+from fedn.common.log_config import logger, set_log_level_from_string, set_log_stream
 from fedn.network.combiner.connect import ConnectorCombiner, Status
 from fedn.network.combiner.modelservice import ModelService
-from fedn.network.combiner.roundhandler import RoundHandler
+from fedn.network.combiner.roundhandler import RoundConfig, RoundHandler
 from fedn.network.grpc.server import Server
 from fedn.network.storage.s3.repository import Repository
 from fedn.network.storage.statestore.mongostatestore import MongoStateStore
 
-VALID_NAME_REGEX = '^[a-zA-Z0-9_-]*$'
+VALID_NAME_REGEX = "^[a-zA-Z0-9_-]*$"
 
 
 class Role(Enum):
-    """ Enum for combiner roles. """
+    """Enum for combiner roles."""
+
     WORKER = 1
     COMBINER = 2
     REDUCER = 3
@@ -33,7 +33,7 @@ class Role(Enum):
 
 
 def role_to_proto_role(role):
-    """ Convert a Role to a proto Role.
+    """Convert a Role to a proto Role.
 
     :param role: the role to convert
     :type role: :class:`fedn.network.combiner.server.Role`
@@ -51,42 +51,41 @@ def role_to_proto_role(role):
 
 
 class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer, rpc.ControlServicer):
-    """ Combiner gRPC server.
+    """Combiner gRPC server.
 
     :param config: configuration for the combiner
     :type config: dict
     """
 
     def __init__(self, config):
-        """ Initialize Combiner server."""
-
-        set_log_level_from_string(config.get('verbosity', "INFO"))
-        set_log_stream(config.get('logfile', None))
+        """Initialize Combiner server."""
+        set_log_level_from_string(config.get("verbosity", "INFO"))
+        set_log_stream(config.get("logfile", None))
 
         # Client queues
         self.clients = {}
 
-        self.modelservice = ModelService()
-
         # Validate combiner name
-        match = re.search(VALID_NAME_REGEX, config['name'])
+        match = re.search(VALID_NAME_REGEX, config["name"])
         if not match:
-            raise ValueError('Unallowed character in combiner name. Allowed characters: a-z, A-Z, 0-9, _, -.')
+            raise ValueError("Unallowed character in combiner name. Allowed characters: a-z, A-Z, 0-9, _, -.")
 
-        self.id = config['name']
+        self.id = config["name"]
         self.role = Role.COMBINER
-        self.max_clients = config['max_clients']
+        self.max_clients = config["max_clients"]
 
         # Connector to announce combiner to discover service (reducer)
-        announce_client = ConnectorCombiner(host=config['discover_host'],
-                                            port=config['discover_port'],
-                                            myhost=config['host'],
-                                            fqdn=config['fqdn'],
-                                            myport=config['port'],
-                                            token=config['token'],
-                                            name=config['name'],
-                                            secure=config['secure'],
-                                            verify=config['verify'])
+        announce_client = ConnectorCombiner(
+            host=config["discover_host"],
+            port=config["discover_port"],
+            myhost=config["host"],
+            fqdn=config["fqdn"],
+            myport=config["port"],
+            token=config["token"],
+            name=config["name"],
+            secure=config["secure"],
+            verify=config["verify"],
+        )
 
         while True:
             # Announce combiner to discover service
@@ -107,27 +106,31 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
                 logger.info("Status.UnMatchedConfig")
                 sys.exit("Exiting: Missing config")
 
-        cert = announce_config['certificate']
-        key = announce_config['key']
+        cert = announce_config["certificate"]
+        key = announce_config["key"]
 
-        if announce_config['certificate']:
-            cert = base64.b64decode(announce_config['certificate'])  # .decode('utf-8')
-            key = base64.b64decode(announce_config['key'])  # .decode('utf-8')
+        if announce_config["certificate"]:
+            cert = base64.b64decode(announce_config["certificate"])  # .decode('utf-8')
+            key = base64.b64decode(announce_config["key"])  # .decode('utf-8')
 
         # Set up gRPC server configuration
-        grpc_config = {'port': config['port'],
-                       'secure': config['secure'],
-                       'certificate': cert,
-                       'key': key}
+        grpc_config = {"port": config["port"], "secure": config["secure"], "certificate": cert, "key": key}
 
         # Set up model repository
-        self.repository = Repository(
-            announce_config['storage']['storage_config'])
+        self.repository = Repository(announce_config["storage"]["storage_config"])
+
+        self.statestore = MongoStateStore(announce_config["statestore"]["network_id"], announce_config["statestore"]["mongo_config"])
+
+        # Fetch all clients previously connected to the combiner
+        # If a client and a combiner goes down at the same time,
+        # the client will be stuck listed as "online" in the statestore.
+        # Set the status to offline for previous clients.
+        previous_clients = self.statestore.clients.find({"combiner": config["name"]})
+        for client in previous_clients:
+            self.statestore.set_client({"name": client["name"], "status": "offline", "client_id": client["client_id"]})
+
+        self.modelservice = ModelService()
 
-        self.statestore = MongoStateStore(
-            announce_config['statestore']['network_id'],
-            announce_config['statestore']['mongo_config']
-        )
         # Create gRPC server
         self.server = Server(self, self.modelservice, grpc_config)
 
@@ -144,7 +147,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         self.server.start()
 
     def __whoami(self, client, instance):
-        """ Set the client id and role in a proto message.
+        """Set the client id and role in a proto message.
 
         :param client: the client to set the id and role for
         :type client: :class:`fedn.network.grpc.fedn_pb2.Client`
@@ -157,8 +160,8 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         client.role = role_to_proto_role(instance.role)
         return client
 
-    def request_model_update(self, config, clients=[]):
-        """ Ask clients to update the current global model.
+    def request_model_update(self, session_id, model_id, config, clients=[]):
+        """Ask clients to update the current global model.
 
         :param config: the model configuration to send to clients
         :type config: dict
@@ -166,35 +169,15 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :type clients: list
 
         """
-        # The request to be added to the client queue
-        request = fedn.TaskRequest()
-        request.model_id = config['model_id']
-        request.correlation_id = str(uuid.uuid4())
-        request.timestamp = str(datetime.now())
-        request.data = json.dumps(config)
-        request.type = fedn.StatusType.MODEL_UPDATE
-        request.session_id = config['session_id']
-
-        request.sender.name = self.id
-        request.sender.role = fedn.COMBINER
-
-        if len(clients) == 0:
-            clients = self.get_active_trainers()
-
-        for client in clients:
-            request.receiver.name = client
-            request.receiver.role = fedn.WORKER
-            self._put_request_to_client_queue(request, fedn.Queue.TASK_QUEUE)
+        clients = self._send_request_type(fedn.StatusType.MODEL_UPDATE, session_id, model_id, config, clients)
 
         if len(clients) < 20:
-            logger.info("Sent model update request for model {} to clients {}".format(
-                request.model_id, clients))
+            logger.info("Sent model update request for model {} to clients {}".format(model_id, clients))
         else:
-            logger.info("Sent model update request for model {} to {} clients".format(
-                request.model_id, len(clients)))
+            logger.info("Sent model update request for model {} to {} clients".format(model_id, len(clients)))
 
-    def request_model_validation(self, model_id, config, clients=[]):
-        """ Ask clients to validate the current global model.
+    def request_model_validation(self, session_id, model_id, clients=[]):
+        """Ask clients to validate the current global model.
 
         :param model_id: the model id to validate
         :type model_id: str
@@ -204,35 +187,77 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :type clients: list
 
         """
-        # The request to be added to the client queue
-        request = fedn.TaskRequest()
-        request.model_id = model_id
-        request.correlation_id = str(uuid.uuid4())
-        request.timestamp = str(datetime.now())
-        # request.is_inference = (config['task'] == 'inference')
-        request.type = fedn.StatusType.MODEL_VALIDATION
-
-        request.sender.name = self.id
-        request.sender.role = fedn.COMBINER
-        request.session_id = config["session_id"]
-
-        if len(clients) == 0:
-            clients = self.get_active_validators()
-
-        for client in clients:
-            request.receiver.name = client
-            request.receiver.role = fedn.WORKER
-            self._put_request_to_client_queue(request, fedn.Queue.TASK_QUEUE)
+        clients = self._send_request_type(fedn.StatusType.MODEL_VALIDATION, session_id, model_id, clients)
 
         if len(clients) < 20:
-            logger.info("Sent model validation request for model {} to clients {}".format(
-                request.model_id, clients))
+            logger.info("Sent model validation request for model {} to clients {}".format(model_id, clients))
         else:
-            logger.info("Sent model validation request for model {} to {} clients".format(
-                request.model_id, len(clients)))
+            logger.info("Sent model validation request for model {} to {} clients".format(model_id, len(clients)))
+
+    def request_model_inference(self, session_id: str, model_id: str, clients: list = []) -> None:
+        """Ask clients to perform inference on the model.
+
+        :param model_id: the model id to perform inference on
+        :type model_id: str
+        :param config: the model configuration to send to clients
+        :type config: dict
+        :param clients: the clients to send the request to
+        :type clients: list
+
+        """
+        clients = self._send_request_type(fedn.StatusType.INFERENCE, session_id, model_id, clients)
+
+        if len(clients) < 20:
+            logger.info("Sent model inference request for model {} to clients {}".format(model_id, clients))
+        else:
+            logger.info("Sent model inference request for model {} to {} clients".format(model_id, len(clients)))
+
+    def _send_request_type(self, request_type, session_id, model_id, config=None, clients=[]):
+        """Send a request of a specific type to clients.
+
+        :param request_type: the type of request
+        :type request_type: :class:`fedn.network.grpc.fedn_pb2.StatusType`
+        :param model_id: the model id to send in the request
+        :type model_id: str
+        :param config: the model configuration to send to clients
+        :type config: dict
+        :param clients: the clients to send the request to
+        :type clients: list
+        :return: the clients
+        :rtype: list
+        """
+        if len(clients) == 0:
+            if request_type == fedn.StatusType.MODEL_UPDATE:
+                clients = self.get_active_trainers()
+            elif request_type == fedn.StatusType.MODEL_VALIDATION:
+                clients = self.get_active_validators()
+            elif request_type == fedn.StatusType.INFERENCE:
+                # TODO: add inference clients type
+                clients = self.get_active_validators()
+        for client in clients:
+            request = fedn.TaskRequest()
+            request.model_id = model_id
+            request.correlation_id = str(uuid.uuid4())
+            request.timestamp = str(datetime.now())
+            request.type = request_type
+            request.session_id = session_id
+
+            request.sender.name = self.id
+            request.sender.role = fedn.COMBINER
+            request.receiver.client_id = client
+            request.receiver.role = fedn.WORKER
+            # Set the request data, not used in validation
+            if request_type == fedn.StatusType.INFERENCE:
+                presigned_url = self.repository.presigned_put_url(self.repository.inference_bucket, f"{client}/{session_id}")
+                # TODO: in inference, request.data should also contain user-defined data/parameters
+                request.data = json.dumps({"presigned_url": presigned_url})
+            elif request_type == fedn.StatusType.MODEL_UPDATE:
+                request.data = json.dumps(config)
+            self._put_request_to_client_queue(request, fedn.Queue.TASK_QUEUE)
+        return clients
 
     def get_active_trainers(self):
-        """ Get a list of active trainers.
+        """Get a list of active trainers.
 
         :return: the list of active trainers
         :rtype: list
@@ -241,7 +266,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return trainers
 
     def get_active_validators(self):
-        """ Get a list of active validators.
+        """Get a list of active validators.
 
         :return: the list of active validators
         :rtype: list
@@ -250,7 +275,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return validators
 
     def nr_active_trainers(self):
-        """ Get the number of active trainers.
+        """Get the number of active trainers.
 
         :return: the number of active trainers
         :rtype: int
@@ -260,17 +285,17 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     ####################################################################################################################
 
     def __join_client(self, client):
-        """ Add a client to the list of active clients.
+        """Add a client to the list of active clients.
 
         :param client: the client to add
         :type client: :class:`fedn.network.grpc.fedn_pb2.Client`
         """
-        if client.name not in self.clients.keys():
+        if client.client_id not in self.clients.keys():
             # The status is set to offline by default, and will be updated once _list_active_clients is called.
-            self.clients[client.name] = {"lastseen": datetime.now(), "status": "offline"}
+            self.clients[client.client_id] = {"last_seen": datetime.now(), "status": "offline"}
 
     def _subscribe_client_to_queue(self, client, queue_name):
-        """ Subscribe a client to the queue.
+        """Subscribe a client to the queue.
 
         :param client: the client to subscribe
         :type client: :class:`fedn.network.grpc.fedn_pb2.Client`
@@ -278,11 +303,11 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :type queue_name: str
         """
         self.__join_client(client)
-        if queue_name not in self.clients[client.name].keys():
-            self.clients[client.name][queue_name] = queue.Queue()
+        if queue_name not in self.clients[client.client_id].keys():
+            self.clients[client.client_id][queue_name] = queue.Queue()
 
     def __get_queue(self, client, queue_name):
-        """ Get the queue for a client.
+        """Get the queue for a client.
 
         :param client: the client to get the queue for
         :type client: :class:`fedn.network.grpc.fedn_pb2.Client`
@@ -294,12 +319,12 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :raises KeyError: if the queue does not exist
         """
         try:
-            return self.clients[client.name][queue_name]
+            return self.clients[client.client_id][queue_name]
         except KeyError:
             raise
 
     def _list_subscribed_clients(self, queue_name):
-        """ List all clients subscribed to a queue.
+        """List all clients subscribed to a queue.
 
         :param queue_name: the name of the queue
         :type queue_name: str
@@ -313,7 +338,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return subscribed_clients
 
     def _list_active_clients(self, channel):
-        """ List all clients that have sent a status message in the last 10 seconds.
+        """List all clients that have sent a status message in the last 10 seconds.
 
         :param channel: the name of the channel
         :type channel: str
@@ -329,18 +354,16 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         for client in self._list_subscribed_clients(channel):
             status = self.clients[client]["status"]
             now = datetime.now()
-            then = self.clients[client]["lastseen"]
+            then = self.clients[client]["last_seen"]
             if (now - then) < timedelta(seconds=10):
                 clients["active_clients"].append(client)
                 # If client has changed status, update statestore
-                if status == "offline":
+                if status != "online":
                     self.clients[client]["status"] = "online"
                     clients["update_active_clients"].append(client)
-            else:
-                # If client has changed status, update statestore
-                if status == "online":
-                    self.clients[client]["status"] = "offline"
-                    clients["update_offline_clients"].append(client)
+            elif status != "offline":
+                self.clients[client]["status"] = "offline"
+                clients["update_offline_clients"].append(client)
         # Update statestore with client status
         if len(clients["update_active_clients"]) > 0:
             self.statestore.update_client_status(clients["update_active_clients"], "online")
@@ -349,15 +372,15 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
 
         return clients["active_clients"]
 
-    def _deamon_thread_client_status(self, timeout=10):
-        """ Deamon thread that checks for inactive clients and updates statestore. """
+    def _deamon_thread_client_status(self, timeout=5):
+        """Deamon thread that checks for inactive clients and updates statestore."""
         while True:
             time.sleep(timeout)
             # TODO: Also update validation clients
             self._list_active_clients(fedn.Queue.TASK_QUEUE)
 
     def _put_request_to_client_queue(self, request, queue_name):
-        """ Get a client specific queue and add a request to it.
+        """Get a client specific queue and add a request to it.
         The client is identified by the request.receiver.
 
         :param request: the request to send
@@ -369,19 +392,15 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             q = self.__get_queue(request.receiver, queue_name)
             q.put(request)
         except Exception as e:
-            logger.error("Failed to put request to client queue {} for client {}: {}".format(
-                queue_name,
-                request.receiver.name,
-                str(e)))
+            logger.error("Failed to put request to client queue {} for client {}: {}".format(queue_name, request.receiver.name, str(e)))
             raise
 
     def _send_status(self, status):
-        """ Report a status to backend db.
+        """Report a status to backend db.
 
         :param status: the status to report
         :type status: :class:`fedn.network.grpc.fedn_pb2.Status`
         """
-
         self.statestore.report_status(status)
 
     def _flush_model_update_queue(self):
@@ -389,7 +408,6 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
 
         :return: True if successful, else False
         """
-
         q = self.round_handler.aggregator.model_updates
         try:
             with q.mutex:
@@ -406,7 +424,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     # Controller Service
 
     def Start(self, control: fedn.ControlRequest, context):
-        """ Start a round of federated learning"
+        """Start a round of federated learning"
 
         :param control: the control request
         :type control: :class:`fedn.network.grpc.fedn_pb2.ControlRequest`
@@ -417,7 +435,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         """
         logger.info("grpc.Combiner.Start: Starting round")
 
-        config = {}
+        config = RoundConfig()
         for parameter in control.parameter:
             config.update({parameter.key: parameter.value})
 
@@ -434,7 +452,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return response
 
     def SetAggregator(self, control: fedn.ControlRequest, context):
-        """ Set the active aggregator.
+        """Set the active aggregator.
 
         :param control: the control request
         :type control: :class:`fedn.network.grpc.fedn_pb2.ControlRequest`
@@ -451,14 +469,14 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
 
         response = fedn.ControlResponse()
         if status:
-            response.message = 'Success'
+            response.message = "Success"
         else:
-            response.message = 'Failed'
+            response.message = "Failed"
 
         return response
 
     def FlushAggregationQueue(self, control: fedn.ControlRequest, context):
-        """ Flush the queue.
+        """Flush the queue.
 
         :param control: the control request
         :type control: :class:`fedn.network.grpc.fedn_pb2.ControlRequest`
@@ -472,16 +490,16 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
 
         response = fedn.ControlResponse()
         if status:
-            response.message = 'Success'
+            response.message = "Success"
         else:
-            response.message = 'Failed'
+            response.message = "Failed"
 
         return response
 
     ##############################################################################
 
     def Stop(self, control: fedn.ControlRequest, context):
-        """ TODO: Not yet implemented.
+        """TODO: Not yet implemented.
 
         :param control: the control request
         :type control: :class:`fedn.network.grpc.fedn_pb2.ControlRequest`
@@ -497,7 +515,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     #####################################################################################################################
 
     def SendStatus(self, status: fedn.Status, context):
-        """ A client RPC endpoint that accepts status messages.
+        """A client RPC endpoint that accepts status messages.
 
         :param status: the status message
         :type status: :class:`fedn.network.grpc.fedn_pb2.Status`
@@ -514,7 +532,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return response
 
     def ListActiveClients(self, request: fedn.ListClientsRequest, context):
-        """ RPC endpoint that returns a ClientList containing the names of all active clients.
+        """RPC endpoint that returns a ClientList containing the names of all active clients.
             An active client has sent a status message / responded to a heartbeat
             request in the last 10 seconds.
 
@@ -538,7 +556,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return clients
 
     def AcceptingClients(self, request: fedn.ConnectionRequest, context):
-        """ RPC endpoint that returns a ConnectionResponse indicating whether the server
+        """RPC endpoint that returns a ConnectionResponse indicating whether the server
         is accepting clients or not.
 
         :param request: the request (unused)
@@ -549,8 +567,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :rtype: :class:`fedn.network.grpc.fedn_pb2.ConnectionResponse`
         """
         response = fedn.ConnectionResponse()
-        active_clients = self._list_active_clients(
-            fedn.Queue.TASK_QUEUE)
+        active_clients = self._list_active_clients(fedn.Queue.TASK_QUEUE)
 
         try:
             requested = int(self.max_clients)
@@ -569,7 +586,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         return response
 
     def SendHeartbeat(self, heartbeat: fedn.Heartbeat, context):
-        """ RPC that lets clients send a hearbeat, notifying the server that
+        """RPC that lets clients send a hearbeat, notifying the server that
             the client is available.
 
         :param heartbeat: the heartbeat
@@ -583,7 +600,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         # Update the clients dict with the last seen timestamp.
         client = heartbeat.sender
         self.__join_client(client)
-        self.clients[client.name]["lastseen"] = datetime.now()
+        self.clients[client.client_id]["last_seen"] = datetime.now()
 
         response = fedn.Response()
         response.sender.name = heartbeat.sender.name
@@ -594,39 +611,40 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     # Combiner Service
 
     def TaskStream(self, response, context):
-        """ A server stream RPC endpoint (Update model). Messages from client stream.
+        """A server stream RPC endpoint (Update model). Messages from client stream.
 
         :param response: the response
         :type response: :class:`fedn.network.grpc.fedn_pb2.ModelUpdateRequest`
         :param context: the context
         :type context: :class:`grpc._server._Context`
         """
-
         client = response.sender
         metadata = context.invocation_metadata()
         if metadata:
             metadata = dict(metadata)
-            logger.info("grpc.Combiner.TaskStream: Client connected: {}\n".format(metadata['client']))
+            logger.info("grpc.Combiner.TaskStream: Client connected: {}\n".format(metadata["client"]))
 
-        status = fedn.Status(
-            status="Client {} connecting to TaskStream.".format(client.name))
+        status = fedn.Status(status="Client {} connecting to TaskStream.".format(client.name))
         status.log_level = fedn.Status.INFO
         status.timestamp.GetCurrentTime()
 
         self.__whoami(status.sender, self)
 
-        self._subscribe_client_to_queue(
-            client, fedn.Queue.TASK_QUEUE)
+        self._subscribe_client_to_queue(client, fedn.Queue.TASK_QUEUE)
         q = self.__get_queue(client, fedn.Queue.TASK_QUEUE)
 
         self._send_status(status)
 
+        # Set client status to online
+        self.clients[client.client_id]["status"] = "online"
+        self.statestore.set_client({"name": client.name, "status": "online", "client_id": client.client_id, "last_seen": datetime.now()})
+
         # Keep track of the time context has been active
         start_time = time.time()
         while context.is_active():
             # Check if the context has been active for more than 10 seconds
             if time.time() - start_time > 10:
-                self.clients[client.name]["lastseen"] = datetime.now()
+                self.clients[client.client_id]["last_seen"] = datetime.now()
                 # Reset the start time
                 start_time = time.time()
             try:
@@ -637,7 +655,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
                 logger.error("Error in ModelUpdateRequestStream: {}".format(e))
 
     def SendModelUpdate(self, request, context):
-        """ Send a model update response.
+        """Send a model update response.
 
         :param request: the request
         :type request: :class:`fedn.network.grpc.fedn_pb2.ModelUpdate`
@@ -649,8 +667,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         self.round_handler.aggregator.on_model_update(request)
 
         response = fedn.Response()
-        response.response = "RECEIVED ModelUpdate {} from client  {}".format(
-            response, response.sender.name)
+        response.response = "RECEIVED ModelUpdate {} from client  {}".format(response, response.sender.name)
         return response  # TODO Fill later
 
     def register_model_validation(self, validation):
@@ -659,11 +676,10 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :param validation: the model validation
         :type validation: :class:`fedn.network.grpc.fedn_pb2.ModelValidation`
         """
-
         self.statestore.report_validation(validation)
 
     def SendModelValidation(self, request, context):
-        """ Send a model validation response.
+        """Send a model validation response.
 
         :param request: the request
         :type request: :class:`fedn.network.grpc.fedn_pb2.ModelValidation`
@@ -677,17 +693,14 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         self.register_model_validation(request)
 
         response = fedn.Response()
-        response.response = "RECEIVED ModelValidation {} from client  {}".format(
-            response, response.sender.name)
+        response.response = "RECEIVED ModelValidation {} from client  {}".format(response, response.sender.name)
         return response
 
     ####################################################################################################################
 
     def run(self):
-        """ Start the server."""
-
-        logger.info("COMBINER: {} started, ready for gRPC requests.".format(
-            self.id))
+        """Start the server."""
+        logger.info("COMBINER: {} started, ready for gRPC requests.".format(self.id))
         try:
             while True:
                 signal.pause()
