@@ -6,6 +6,7 @@ import pymongo
 from google.protobuf.json_format import MessageToDict
 
 from fedn.common.log_config import logger
+from fedn.network.combiner.roundhandler import RoundConfig
 from fedn.network.state import ReducerStateToString, StringToReducerState
 
 
@@ -61,7 +62,7 @@ class MongoStateStore:
         self.init_index()
 
     def connect(self):
-        """ Establish client connection to MongoDB.
+        """Establish client connection to MongoDB.
 
         :param config: Dictionary containing connection strings and security credentials.
         :type config: dict
@@ -80,6 +81,7 @@ class MongoStateStore:
 
     def init_index(self):
         self.package.create_index([("id", pymongo.DESCENDING)])
+        self.clients.create_index([("client_id", pymongo.DESCENDING)])
 
     def is_inited(self):
         """Check if the statestore is intialized.
@@ -125,11 +127,7 @@ class MongoStateStore:
                 True,
             )
         else:
-            logger.info(
-                "Not updating state, already in {}".format(
-                    ReducerStateToString(state)
-                )
-            )
+            logger.info("Not updating state, already in {}".format(ReducerStateToString(state)))
 
     def get_sessions(self, limit=None, skip=None, sort_key="_id", sort_order=pymongo.DESCENDING):
         """Get all sessions.
@@ -144,20 +142,15 @@ class MongoStateStore:
         :type sort_order: pymongo.ASCENDING or pymongo.DESCENDING
         :return: Dictionary of sessions in result (array of session objects) and count.
         """
-
         result = None
 
         if limit is not None and skip is not None:
             limit = int(limit)
             skip = int(skip)
 
-            result = self.sessions.find().limit(limit).skip(skip).sort(
-                sort_key, sort_order
-            )
+            result = self.sessions.find().limit(limit).skip(skip).sort(sort_key, sort_order)
         else:
-            result = self.sessions.find().sort(
-                sort_key, sort_order
-            )
+            result = self.sessions.find().sort(sort_key, sort_order)
 
         count = self.sessions.count_documents({})
 
@@ -176,6 +169,17 @@ class MongoStateStore:
         """
         return self.sessions.find_one({"session_id": session_id})
 
+    def get_session_status(self, session_id):
+        """Get the session status.
+
+        :param session_id: The session id.
+        :type session_id: str
+        :return: The session status.
+        :rtype: str
+        """
+        session = self.sessions.find_one({"session_id": session_id})
+        return session["status"]
+
     def set_latest_model(self, model_id, session_id=None):
         """Set the latest model id.
 
@@ -183,7 +187,6 @@ class MongoStateStore:
         :type model_id: str
         :return:
         """
-
         committed_at = datetime.now()
         current_model = self.model.find_one({"key": "current_model"})
         parent_model = None
@@ -204,9 +207,7 @@ class MongoStateStore:
             }
         )
 
-        self.model.update_one(
-            {"key": "current_model"}, {"$set": {"model": model_id}}, True
-        )
+        self.model.update_one({"key": "current_model"}, {"$set": {"model": model_id}}, True)
         self.model.update_one(
             {"key": "model_trail"},
             {
@@ -224,10 +225,7 @@ class MongoStateStore:
         :return: The initial model id. None if no model is found.
         :rtype: str
         """
-
-        result = self.model.find_one(
-            {"key": "model_trail"}, sort=[("committed_at", pymongo.ASCENDING)]
-        )
+        result = self.model.find_one({"key": "model_trail"}, sort=[("committed_at", pymongo.ASCENDING)])
         if result is None:
             return None
 
@@ -264,18 +262,13 @@ class MongoStateStore:
         :type model_id: str
         :return:
         """
-
         try:
-
             committed_at = datetime.now()
 
             existing_model = self.model.find_one({"key": "models", "model": model_id})
 
             if existing_model is not None:
-
-                self.model.update_one(
-                    {"key": "current_model"}, {"$set": {"model": model_id, "committed_at": committed_at, "session_id": None}}, True
-                )
+                self.model.update_one({"key": "current_model"}, {"$set": {"model": model_id, "committed_at": committed_at, "session_id": None}}, True)
 
                 return True
         except Exception as e:
@@ -289,7 +282,6 @@ class MongoStateStore:
         :return: The id of the most recent round.
         :rtype: ObjectId
         """
-
         return self.rounds.find_one(sort=[("_id", pymongo.DESCENDING)])
 
     def get_round(self, id):
@@ -300,7 +292,6 @@ class MongoStateStore:
         :return: round with id, reducer and combiners
         :rtype: ObjectId
         """
-
         return self.rounds.find_one({"round_id": str(id)})
 
     def get_rounds(self):
@@ -309,7 +300,6 @@ class MongoStateStore:
         :return: All rounds.
         :rtype: ObjectId
         """
-
         return self.rounds.find()
 
     def get_validations(self, **kwargs):
@@ -320,7 +310,6 @@ class MongoStateStore:
         :return: validations matching query
         :rtype: ObjectId
         """
-
         result = self.control.validations.find(kwargs)
         return result
 
@@ -332,9 +321,7 @@ class MongoStateStore:
         :return: True if successful.
         :rtype: bool
         """
-
         try:
-
             find = {"id": id}
             projection = {"_id": False, "key": False}
 
@@ -345,9 +332,7 @@ class MongoStateStore:
 
             doc["key"] = "active"
 
-            self.control.package.replace_one(
-                {"key": "active"}, doc
-            )
+            self.control.package.replace_one({"key": "active"}, doc)
 
         except Exception as e:
             logger.error("ERROR: {}".format(e))
@@ -363,7 +348,6 @@ class MongoStateStore:
         :return: True if successful.
         :rtype: bool
         """
-
         obj = {
             "file_name": file_name,
             "storage_file_name": storage_file_name,
@@ -376,9 +360,7 @@ class MongoStateStore:
 
         self.control.package.update_one(
             {"key": "active"},
-            {
-                "$set": obj
-            },
+            {"$set": obj},
             True,
         )
 
@@ -395,7 +377,6 @@ class MongoStateStore:
         :rtype: ObjectID
         """
         try:
-
             find = {"key": "active"}
             projection = {"key": False, "_id": False}
             ret = self.control.package.find_one(find, projection)
@@ -418,7 +399,6 @@ class MongoStateStore:
         :return: Dictionary of compute packages in result and count.
         :rtype: dict
         """
-
         result = None
         count = None
 
@@ -449,9 +429,7 @@ class MongoStateStore:
         :type helper: str
         :return:
         """
-        self.control.package.update_one(
-            {"key": "active"}, {"$set": {"helper": helper}}, True
-        )
+        self.control.package.update_one({"key": "active"}, {"$set": {"helper": helper}}, True)
 
     def get_helper(self):
         """Get the active helper package.
@@ -466,9 +444,7 @@ class MongoStateStore:
         #    ret = self.control.config.find_one({'key': 'round_config'})
         try:
             retcheck = ret["helper"]
-            if (
-                retcheck == "" or retcheck == " "
-            ):  # ugly check for empty string
+            if retcheck == "" or retcheck == " ":  # ugly check for empty string
                 return None
             return retcheck
         except (KeyError, IndexError):
@@ -495,11 +471,7 @@ class MongoStateStore:
         """
         result = None
 
-        find_option = (
-            {"key": "models"}
-            if session_id is None
-            else {"key": "models", "session_id": session_id}
-        )
+        find_option = {"key": "models"} if session_id is None else {"key": "models", "session_id": session_id}
 
         projection = {"_id": False, "key": False}
 
@@ -507,17 +479,10 @@ class MongoStateStore:
             limit = int(limit)
             skip = int(skip)
 
-            result = (
-                self.model.find(find_option, projection)
-                .limit(limit)
-                .skip(skip)
-                .sort(sort_key, sort_order)
-            )
+            result = self.model.find(find_option, projection).limit(limit).skip(skip).sort(sort_key, sort_order)
 
         else:
-            result = self.model.find(find_option, projection).sort(
-                sort_key, sort_order
-            )
+            result = self.model.find(find_option, projection).sort(sort_key, sort_order)
 
         count = self.model.count_documents(find_option)
 
@@ -581,7 +546,6 @@ class MongoStateStore:
         :return: List of model descendants.
         :rtype: list
         """
-
         model: object = self.model.find_one({"key": "models", "model": model_id})
         current_model_id: str = model["model"] if model is not None else None
         result: list = []
@@ -625,9 +589,7 @@ class MongoStateStore:
         projection = {"_id": False}
 
         if not kwargs:
-            result = self.control.status.find({}, projection).sort(
-                "timestamp", pymongo.DESCENDING
-            )
+            result = self.control.status.find({}, projection).sort("timestamp", pymongo.DESCENDING)
             count = self.control.status.count_documents({})
         else:
             limit = kwargs.pop("limit", None)
@@ -636,16 +598,9 @@ class MongoStateStore:
             if limit is not None and skip is not None:
                 limit = int(limit)
                 skip = int(skip)
-                result = (
-                    self.control.status.find(kwargs, projection)
-                    .sort("timestamp", pymongo.DESCENDING)
-                    .limit(limit)
-                    .skip(skip)
-                )
+                result = self.control.status.find(kwargs, projection).sort("timestamp", pymongo.DESCENDING).limit(limit).skip(skip)
             else:
-                result = self.control.status.find(kwargs, projection).sort(
-                    "timestamp", pymongo.DESCENDING
-                )
+                result = self.control.status.find(kwargs, projection).sort("timestamp", pymongo.DESCENDING)
 
             count = self.control.status.count_documents(kwargs)
 
@@ -661,9 +616,7 @@ class MongoStateStore:
         :rtype: ObjectID
         """
         try:
-            ret = self.storage.find(
-                {"status": "enabled"}, projection={"_id": False}
-            )
+            ret = self.storage.find({"status": "enabled"}, projection={"_id": False})
             return ret[0]
         except (KeyError, IndexError):
             return None
@@ -678,9 +631,7 @@ class MongoStateStore:
         config = copy.deepcopy(config)
         config["updated_at"] = str(datetime.now())
         config["status"] = "enabled"
-        self.storage.update_one(
-            {"storage_type": config["storage_type"]}, {"$set": config}, True
-        )
+        self.storage.update_one({"storage_type": config["storage_type"]}, {"$set": config}, True)
 
     def set_reducer(self, reducer_data):
         """Set the reducer in the statestore.
@@ -690,9 +641,7 @@ class MongoStateStore:
         :return:
         """
         reducer_data["updated_at"] = str(datetime.now())
-        self.reducer.update_one(
-            {"name": reducer_data["name"]}, {"$set": reducer_data}, True
-        )
+        self.reducer.update_one({"name": reducer_data["name"]}, {"$set": reducer_data}, True)
 
     def get_reducer(self):
         """Get reducer.config.
@@ -736,7 +685,6 @@ class MongoStateStore:
         :return: Dictionary of combiners in result and count.
         :rtype: dict
         """
-
         result = None
         count = None
 
@@ -765,11 +713,8 @@ class MongoStateStore:
         :type combiner_data: dict
         :return:
         """
-
         combiner_data["updated_at"] = str(datetime.now())
-        self.combiners.update_one(
-            {"name": combiner_data["name"]}, {"$set": combiner_data}, True
-        )
+        self.combiners.update_one({"name": combiner_data["name"]}, {"$set": combiner_data}, True)
 
     def delete_combiner(self, combiner):
         """Delete a combiner from statestore.
@@ -793,20 +738,18 @@ class MongoStateStore:
         :return:
         """
         client_data["updated_at"] = str(datetime.now())
-        self.clients.update_one(
-            {"name": client_data["name"]}, {"$set": client_data}, True
-        )
+        self.clients.update_one({"client_id": client_data["client_id"]}, {"$set": client_data}, True)
 
-    def get_client(self, name):
-        """Get client by name.
+    def get_client(self, client_id):
+        """Get client by client_id.
 
-        :param name: name of client to get.
-        :type name: str
+        :param client_id: client_id of client to get.
+        :type client_id: str
         :return: The client. None if not found.
         :rtype: ObjectId
         """
         try:
-            ret = self.clients.find({"key": name})
+            ret = self.clients.find({"key": client_id})
             if list(ret) == []:
                 return None
             else:
@@ -825,7 +768,6 @@ class MongoStateStore:
         :type status: str
         :param sort_key: The key to sort by.
         """
-
         result = None
         count = None
 
@@ -862,19 +804,18 @@ class MongoStateStore:
         :return: list of combiner data.
         :rtype: list(ObjectId)
         """
-
         result = None
 
         try:
-
-            pipeline = [
-                {"$match": {"combiner": {"$in": combiners}, "status": "online"}},
-                {"$group": {"_id": "$combiner", "count": {"$sum": 1}}},
-                {"$sort": {sort_key: sort_order, "_id": pymongo.ASCENDING}}
-            ] if combiners is not None else [
-                {"$group": {"_id": "$combiner", "count": {"$sum": 1}}},
-                {"$sort": {sort_key: sort_order, "_id": pymongo.ASCENDING}}
-            ]
+            pipeline = (
+                [
+                    {"$match": {"combiner": {"$in": combiners}, "status": "online"}},
+                    {"$group": {"_id": "$combiner", "count": {"$sum": 1}}},
+                    {"$sort": {sort_key: sort_order, "_id": pymongo.ASCENDING}},
+                ]
+                if combiners is not None
+                else [{"$group": {"_id": "$combiner", "count": {"$sum": 1}}}, {"$sort": {sort_key: sort_order, "_id": pymongo.ASCENDING}}]
+            )
 
             result = self.clients.aggregate(pipeline)
 
@@ -911,7 +852,7 @@ class MongoStateStore:
             self.status.drop()
 
     def create_session(self, id=None):
-        """ Create a new session object.
+        """Create a new session object.
 
         :param id: The ID of the created session.
         :type id: uuid, str
@@ -919,11 +860,11 @@ class MongoStateStore:
         """
         if not id:
             id = uuid.uuid4()
-        data = {'session_id': str(id)}
+        data = {"session_id": str(id)}
         self.sessions.insert_one(data)
 
     def create_round(self, round_data):
-        """ Create a new round.
+        """Create a new round.
 
         :param round_data: Dictionary with round data.
         :type round_data: dict
@@ -931,7 +872,7 @@ class MongoStateStore:
         # TODO: Add check if round_id already exists
         self.rounds.insert_one(round_data)
 
-    def set_session_config(self, id, config):
+    def set_session_config(self, id: str, config: RoundConfig) -> None:
         """Set the session configuration.
 
         :param id: The session id
@@ -939,8 +880,18 @@ class MongoStateStore:
         :param config: Session configuration
         :type config: dict
         """
-        self.sessions.update_one({'session_id': str(id)}, {
-            '$push': {'session_config': config}}, True)
+        self.sessions.update_one({"session_id": str(id)}, {"$push": {"session_config": config}}, True)
+
+    # Added to accomodate new session config structure
+    def set_session_config_v2(self, id: str, config: RoundConfig) -> None:
+        """Set the session configuration.
+
+        :param id: The session id
+        :type id: str
+        :param config: Session configuration
+        :type config: dict
+        """
+        self.sessions.update_one({"session_id": str(id)}, {"$set": {"session_config": config}}, True)
 
     def set_session_status(self, id, status):
         """Set session status.
@@ -949,8 +900,7 @@ class MongoStateStore:
         :type round_id: str
         :param round_status: The status of the session.
         """
-        self.sessions.update_one({'session_id': str(id)}, {
-            '$set': {'status': status}}, True)
+        self.sessions.update_one({"session_id": str(id)}, {"$set": {"status": status}}, True)
 
     def set_round_combiner_data(self, data):
         """Set combiner round controller data.
@@ -958,10 +908,9 @@ class MongoStateStore:
         :param data: The combiner data
         :type data: dict
         """
-        self.rounds.update_one({'round_id': str(data['round_id'])}, {
-            '$push': {'combiners': data}}, True)
+        self.rounds.update_one({"round_id": str(data["round_id"])}, {"$push": {"combiners": data}}, True)
 
-    def set_round_config(self, round_id, round_config):
+    def set_round_config(self, round_id, round_config: RoundConfig):
         """Set round configuration.
 
         :param round_id: The round unique identifier
@@ -969,8 +918,7 @@ class MongoStateStore:
         :param round_config: The round configuration
         :type round_config: dict
         """
-        self.rounds.update_one({'round_id': round_id}, {
-            '$set': {'round_config': round_config}}, True)
+        self.rounds.update_one({"round_id": round_id}, {"$set": {"round_config": round_config}}, True)
 
     def set_round_status(self, round_id, round_status):
         """Set round status.
@@ -979,8 +927,7 @@ class MongoStateStore:
         :type round_id: str
         :param round_status: The status of the round.
         """
-        self.rounds.update_one({'round_id': round_id}, {
-            '$set': {'status': round_status}}, True)
+        self.rounds.update_one({"round_id": round_id}, {"$set": {"status": round_status}}, True)
 
     def set_round_data(self, round_id, round_data):
         """Update round metadata
@@ -990,11 +937,10 @@ class MongoStateStore:
         :param round_data: The round metadata
         :type round_data: dict
         """
-        self.rounds.update_one({'round_id': round_id}, {
-            '$set': {'round_data': round_data}}, True)
+        self.rounds.update_one({"round_id": round_id}, {"$set": {"round_data": round_data}}, True)
 
     def update_client_status(self, clients, status):
-        """ Update client status in statestore.
+        """Update client status in statestore.
         :param client_name: The client name
         :type client_name: str
         :param status: The client status
@@ -1002,7 +948,7 @@ class MongoStateStore:
         :return: None
         """
         datetime_now = datetime.now()
-        filter_query = {"name": {"$in": clients}}
+        filter_query = {"client_id": {"$in": clients}}
 
         update_query = {"$set": {"last_seen": datetime_now, "status": status}}
         self.clients.update_many(filter_query, update_query)
