@@ -1,7 +1,7 @@
 
 def from_document(document: dict) -> dict:
-    document['id'] = str(document['_id'])
-    del document['_id']
+    document["id"] = str(document["_id"])
+    del document["_id"]
     return document
 
 
